/*

  bootloader - let bprom transfer code to single board computer 
  Nils Eilers <nils.eilers@gmx.de>

  first edit: 2008-06-24
  last edit : 2008-06-24
*/

#include <stdio.h>   	/* Standard input/output definitions */
#include <string.h>  	/* String function definitions */
#include <unistd.h>  	/* UNIX standard function definitions */
#include <fcntl.h>   	/* File control definitions */
#include <errno.h>   	/* Error number definitions */
#include <termios.h> 	/* POSIX terminal control definitions */
#include <stdlib.h>	/* exit() */
#include <stdarg.h>	/* variable arguments */
#include <sys/stat.h>	/* file permissions */

#include <cmdline.h>
#include <bprom.h>
#include <bootloader.h>


int bootloader(struct gengetopt_args_info *args_info, int argc)
{
  lvprintf(VB_DBG, "programmer model: bootloader\n");
  
  lvprintf(VB_ERR, "not yet implemented\n"); return 3;

  if(args_info->size_given) {
    size=args_info->size_arg;
  }
  if(args_info->from_given) {
    from=args_info->from_arg;
  }
  
  dump_args();
  fd=open_port(FALSE);
  
  return close_port(FALSE); 
}

int read_sbc(long from, long size)
{
  int rd, crc, crc_soll, errcde;
  long sum; 
  HML hml_from; HML hml_size;
  char cmdbuffer[16];
  
  lvprintf(VB_DBG, "read_eprom: ictyp=%02X from=%lX size=%lX\n", \
                                            ictyp, from, size);
  long2hml(from, &hml_from); long2hml(size, &hml_size);  
  
  cmdbuffer[0]=0;
  cmdbuffer[1]=0;
  cmdbuffer[2]=hml_from.h;
  cmdbuffer[3]=hml_from.m;
  cmdbuffer[4]=hml_from.l;
  cmdbuffer[5]=hml_size.h;
  cmdbuffer[6]=hml_size.m;
  cmdbuffer[7]=hml_size.l;
  show_cmdbuffer(cmdbuffer, 8);
  send_cmdbuffer(cmdbuffer, 8);  

  lvprintf(VB_DBG, "Waiting for result...\n");
  do {
    rd=read(fd, buffer, 1);
  } while(!rd);

  errcde=buffer[0];
  lvprintf(VB_DBG, "Result: %X\n", errcde);
    
  if(errcde) {
    return(errcde);    
  } else
  {
    lvprintf(VB_DBG, "Reading data...\n");
    sum=0;
    while(sum<(size+1)) {
      statln(sum, size);
      rd=read(fd, buffer+from+sum, size+1-sum);
      if(rd<0) {
        perror("reading serial data");
        break;
      }
      sum+=rd;
      lvprintf(VB_DBG, "%d bytes received, total %ld of %ld+1.\n", rd, sum, size);
    }
    endstatln(size, size); 

    show_buffer(from, size);

    crc_soll=buffer[from+size] & 0xFF;
    lvprintf(VB_DBG, "CRC (prommer): %02X\n", crc_soll);

    crc=calc_crc(from, size);

    lvprintf(VB_DBG, (crc==crc_soll) ? "CRC OK\n" : "CRC ERROR\n");
    if(crc==crc_soll) return 0;
    else return -1;
  } /* if(errcde) */
}


int prg_1buffer_sbc(long from, long size)
{
  int rd, wr, crc, errcde;
  long sum; 
  HML hml_from; HML hml_size; HML ea;
  char cmdbuffer[16];
  char rcbuffer[4];
  
  lvprintf(VB_DBG, "prg_1buffer: from=%lXh size=%lXh\n", from, size);
  
  long2hml(from, &hml_from); long2hml(size, &hml_size);  
  erraddr=0;
  
  crc=calc_crc(from, size);
  cmdbuffer[8]=crc;
  
  cmdbuffer[0]=CMD_PRG_COPY;
  cmdbuffer[1]=0;
  cmdbuffer[2]=hml_from.h;
  cmdbuffer[3]=hml_from.m;
  cmdbuffer[4]=hml_from.l;
  cmdbuffer[5]=hml_size.h;
  cmdbuffer[6]=hml_size.m;
  cmdbuffer[7]=hml_size.l;
  show_cmdbuffer(cmdbuffer, 8);
  send_cmdbuffer(cmdbuffer, 8);  
  
  wr=write(fd, buffer+from, size);		/* programm data */
  
  write(fd, cmdbuffer+8, 1);  			/* crc */

  lvprintf(VB_DBG, "Waiting for result...\n");
  do {
    rd=read(fd, rcbuffer, 1);
  } while(!rd);

  errcde=rcbuffer[0] & 0xFF;
  lvprintf(VB_DBG, "Result: %X\n", errcde);
  if(errcde) return(errcde);
  
  lvprintf(VB_DBG, "Waiting for result...\n");
  do {
    rd=read(fd, rcbuffer, 1);
  } while(!rd);

  errcde=rcbuffer[0] & 0xFF;
  lvprintf(VB_DBG, "Result: %X\n", errcde);
    
  if(errcde) {				/* failed */
    switch(errcde) {
      case 0x92:
      case 0x93:
      case 0x94:
                                        /* reading address */
        sum=0;				/* where error occured */
        while(sum<3) {
          rd=read(fd, rcbuffer+sum, 3-sum);
          sum+=rd;
        }
        ea.h=rcbuffer[0];
        ea.m=rcbuffer[1];
        ea.l=rcbuffer[2];
        erraddr=hml2long(&ea);
        break;
        
      default:
        break;
    } /* switch(errcde) */
  } /* if (errcde) */
  return(errcde);    
}

int write_sbc(long from, long size)
{
  int errcde;			/* error code */
  long current_size; 		/* bytes to be written this time */
  long sum;			/* bytes already written */
  long total;			/* bytes yet to be written */
  
  lvprintf(VB_DBG, "prg_copy: from=%lX size=%lX\n", from, size);

  sum=0; total=size;  
  do {
    current_size=total;
    if(current_size>PROMBUFSIZE) {
      total-=PROMBUFSIZE;
      current_size=PROMBUFSIZE;
    }
    lvprintf(LOG_INFO, \
    "Programming %lXh bytes starting at %lXh\n", current_size, from+sum);
    statln(sum, size);
    errcde=prg_1buffer_sbc(from+sum, current_size);
    if(errcde) {
      statln(-1, 0);
      return errcde;
    }
    sum+=current_size;
  } while(sum<size);    
  endstatln(size, size);
  return errcde;
}

